@@ -1,29 +1,28 @@
 """Denon HEOS Media Player."""
 from __future__ import annotations
 
+from collections.abc import Awaitable, Callable, Coroutine
 from functools import reduce, wraps
 import logging
 from operator import ior
+from typing import Any
 
 from pyheos import HeosError, const as heos_const
+from typing_extensions import ParamSpec
 
 from homeassistant.components import media_source
 from homeassistant.components.media_player import (
-    MediaPlayerEntity,
-    MediaPlayerEntityFeature,
-)
-from homeassistant.components.media_player.browse_media import (
-    async_process_play_media_url,
-)
-from homeassistant.components.media_player.const import (
     ATTR_MEDIA_ENQUEUE,
     DOMAIN,
-    MEDIA_TYPE_MUSIC,
-    MEDIA_TYPE_PLAYLIST,
-    MEDIA_TYPE_URL,
+    BrowseMedia,
+    MediaPlayerEnqueue,
+    MediaPlayerEntity,
+    MediaPlayerEntityFeature,
+    MediaPlayerState,
+    MediaType,
+    async_process_play_media_url,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import STATE_IDLE, STATE_PAUSED, STATE_PLAYING
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
@@ -42,6 +41,8 @@ from .const import (
     SIGNAL_HEOS_UPDATED,
 )
 
+_P = ParamSpec("_P")
+
 BASE_SUPPORTED_FEATURES = (
     MediaPlayerEntityFeature.VOLUME_MUTE
     | MediaPlayerEntityFeature.VOLUME_SET
@@ -55,9 +56,9 @@ BASE_SUPPORTED_FEATURES = (
 )
 
 PLAY_STATE_TO_STATE = {
-    heos_const.PLAY_STATE_PLAY: STATE_PLAYING,
-    heos_const.PLAY_STATE_STOP: STATE_IDLE,
-    heos_const.PLAY_STATE_PAUSE: STATE_PAUSED,
+    heos_const.PLAY_STATE_PLAY: MediaPlayerState.PLAYING,
+    heos_const.PLAY_STATE_STOP: MediaPlayerState.IDLE,
+    heos_const.PLAY_STATE_PAUSE: MediaPlayerState.PAUSED,
 }
 
 CONTROL_TO_SUPPORT = {
@@ -68,6 +69,14 @@ CONTROL_TO_SUPPORT = {
     heos_const.CONTROL_PLAY_NEXT: MediaPlayerEntityFeature.NEXT_TRACK,
 }
 
+HA_HEOS_ENQUEUE_MAP = {
+    None: heos_const.ADD_QUEUE_REPLACE_AND_PLAY,
+    MediaPlayerEnqueue.ADD: heos_const.ADD_QUEUE_ADD_TO_END,
+    MediaPlayerEnqueue.REPLACE: heos_const.ADD_QUEUE_REPLACE_AND_PLAY,
+    MediaPlayerEnqueue.NEXT: heos_const.ADD_QUEUE_PLAY_NEXT,
+    MediaPlayerEnqueue.PLAY: heos_const.ADD_QUEUE_PLAY_NOW,
+}
+
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -80,12 +89,16 @@ async def async_setup_entry(
     async_add_entities(devices, True)
 
 
-def log_command_error(command: str):
+def log_command_error(
+    command: str,
+) -> Callable[[Callable[_P, Awaitable[Any]]], Callable[_P, Coroutine[Any, Any, None]]]:
     """Return decorator that logs command failure."""
 
-    def decorator(func):
+    def decorator(
+        func: Callable[_P, Awaitable[Any]]
+    ) -> Callable[_P, Coroutine[Any, Any, None]]:
         @wraps(func)
-        async def wrapper(*args, **kwargs):
+        async def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> None:
             try:
                 await func(*args, **kwargs)
             except (HeosError, ValueError) as ex:
@@ -99,6 +112,9 @@ def log_command_error(command: str):
 class HeosMediaPlayer(MediaPlayerEntity):
     """The HEOS player."""
 
+    _attr_media_content_type = MediaType.MUSIC
+    _attr_should_poll = False
+
     def __init__(self, player):
         """Initialize."""
         self._media_position_updated_at = None
@@ -116,11 +132,11 @@ class HeosMediaPlayer(MediaPlayerEntity):
             self._media_position_updated_at = utcnow()
         await self.async_update_ha_state(True)
 
-    async def _heos_updated(self):
+    async def _heos_updated(self) -> None:
         """Handle sources changed."""
         await self.async_update_ha_state(True)
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Device added to hass."""
         # Update state when attributes of the player change
         self._signals.append(
@@ -139,7 +155,7 @@ class HeosMediaPlayer(MediaPlayerEntity):
         async_dispatcher_send(self.hass, SIGNAL_HEOS_PLAYER_ADDED)
 
     @log_command_error("clear playlist")
-    async def async_clear_playlist(self):
+    async def async_clear_playlist(self) -> None:
         """Clear players playlist."""
         await self._player.clear_queue()
 
@@ -149,44 +165,48 @@ class HeosMediaPlayer(MediaPlayerEntity):
         await self._group_manager.async_join_players(self.entity_id, group_members)
 
     @log_command_error("pause")
-    async def async_media_pause(self):
+    async def async_media_pause(self) -> None:
         """Send pause command."""
         await self._player.pause()
 
     @log_command_error("play")
-    async def async_media_play(self):
+    async def async_media_play(self) -> None:
         """Send play command."""
         await self._player.play()
 
     @log_command_error("move to previous track")
-    async def async_media_previous_track(self):
+    async def async_media_previous_track(self) -> None:
         """Send previous track command."""
         await self._player.play_previous()
 
     @log_command_error("move to next track")
-    async def async_media_next_track(self):
+    async def async_media_next_track(self) -> None:
         """Send next track command."""
         await self._player.play_next()
 
     @log_command_error("stop")
-    async def async_media_stop(self):
+    async def async_media_stop(self) -> None:
         """Send stop command."""
         await self._player.stop()
 
     @log_command_error("set mute")
-    async def async_mute_volume(self, mute):
+    async def async_mute_volume(self, mute: bool) -> None:
         """Mute the volume."""
         await self._player.set_mute(mute)
 
     @log_command_error("play media")
-    async def async_play_media(self, media_type, media_id, **kwargs):
+    async def async_play_media(
+        self, media_type: str, media_id: str, **kwargs: Any
+    ) -> None:
         """Play a piece of media."""
         if media_source.is_media_source_id(media_id):
-            media_type = MEDIA_TYPE_URL
-            play_item = await media_source.async_resolve_media(self.hass, media_id)
+            media_type = MediaType.URL
+            play_item = await media_source.async_resolve_media(
+                self.hass, media_id, self.entity_id
+            )
             media_id = play_item.url
 
-        if media_type in (MEDIA_TYPE_URL, MEDIA_TYPE_MUSIC):
+        if media_type in {MediaType.URL, MediaType.MUSIC}:
             media_id = async_process_play_media_url(self.hass, media_id)
 
             await self._player.play_url(media_id)
@@ -196,7 +216,7 @@ class HeosMediaPlayer(MediaPlayerEntity):
             # media_id may be an int or a str
             selects = await self._player.get_quick_selects()
             try:
-                index = int(media_id)
+                index: int | None = int(media_id)
             except ValueError:
                 # Try finding index by name
                 index = next(
@@ -208,16 +228,13 @@ class HeosMediaPlayer(MediaPlayerEntity):
             await self._player.play_quick_select(index)
             return
 
-        if media_type == MEDIA_TYPE_PLAYLIST:
+        if media_type == MediaType.PLAYLIST:
             playlists = await self._player.heos.get_playlists()
             playlist = next((p for p in playlists if p.name == media_id), None)
             if not playlist:
                 raise ValueError(f"Invalid playlist '{media_id}'")
-            add_queue_option = (
-                heos_const.ADD_QUEUE_ADD_TO_END
-                if kwargs.get(ATTR_MEDIA_ENQUEUE)
-                else heos_const.ADD_QUEUE_REPLACE_AND_PLAY
-            )
+            add_queue_option = HA_HEOS_ENQUEUE_MAP.get(kwargs.get(ATTR_MEDIA_ENQUEUE))
+
             await self._player.add_to_queue(playlist, add_queue_option)
             return
 
@@ -243,21 +260,21 @@ class HeosMediaPlayer(MediaPlayerEntity):
         raise ValueError(f"Unsupported media type '{media_type}'")
 
     @log_command_error("select source")
-    async def async_select_source(self, source):
+    async def async_select_source(self, source: str) -> None:
         """Select input source."""
         await self._source_manager.play_source(source, self._player)
 
     @log_command_error("set shuffle")
-    async def async_set_shuffle(self, shuffle):
+    async def async_set_shuffle(self, shuffle: bool) -> None:
         """Enable/disable shuffle mode."""
         await self._player.set_play_mode(self._player.repeat, shuffle)
 
     @log_command_error("set volume level")
-    async def async_set_volume_level(self, volume):
+    async def async_set_volume_level(self, volume: float) -> None:
         """Set volume level, range 0..1."""
         await self._player.set_volume(int(volume * 100))
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Update supported features of the player."""
         controls = self._player.now_playing_media.supported_controls
         current_support = [CONTROL_TO_SUPPORT[control] for control in controls]
@@ -272,11 +289,11 @@ class HeosMediaPlayer(MediaPlayerEntity):
             self._source_manager = self.hass.data[HEOS_DOMAIN][DATA_SOURCE_MANAGER]
 
     @log_command_error("unjoin_player")
-    async def async_unjoin_player(self):
+    async def async_unjoin_player(self) -> None:
         """Remove this player from any group."""
         await self._group_manager.async_unjoin_player(self.entity_id)
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Disconnect the device when removed."""
         for signal_remove in self._signals:
             signal_remove()
@@ -299,7 +316,7 @@ class HeosMediaPlayer(MediaPlayerEntity):
         )
 
     @property
-    def extra_state_attributes(self) -> dict:
+    def extra_state_attributes(self) -> dict[str, Any]:
         """Get additional attribute about the state."""
         return {
             "media_album_id": self._player.now_playing_media.album_id,
@@ -334,11 +351,6 @@ class HeosMediaPlayer(MediaPlayerEntity):
         """Content ID of current playing media."""
         return self._player.now_playing_media.media_id
 
-    @property
-    def media_content_type(self) -> str:
-        """Content type of current playing media."""
-        return MEDIA_TYPE_MUSIC
-
     @property
     def media_duration(self):
         """Duration of current playing media in seconds."""
@@ -385,11 +397,6 @@ class HeosMediaPlayer(MediaPlayerEntity):
         """Return the name of the device."""
         return self._player.name
 
-    @property
-    def should_poll(self) -> bool:
-        """No polling needed for this device."""
-        return False
-
     @property
     def shuffle(self) -> bool:
         """Boolean if shuffle is enabled."""
@@ -420,7 +427,9 @@ class HeosMediaPlayer(MediaPlayerEntity):
         """Volume level of the media player (0..1)."""
         return self._player.volume / 100
 
-    async def async_browse_media(self, media_content_type=None, media_content_id=None):
+    async def async_browse_media(
+        self, media_content_type: str | None = None, media_content_id: str | None = None
+    ) -> BrowseMedia:
         """Implement the websocket media browsing helper."""
         return await media_source.async_browse_media(
             self.hass,
