@@ -8,15 +8,19 @@ from homeassistant.components.sensor import (
     SensorEntityDescription,
     SensorStateClass,
 )
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import (
+    DATA_GIGABYTES,
+    DATA_MEGABYTES,
+    DATA_RATE_MEGABYTES_PER_SECOND,
+)
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.device_registry import DeviceEntryType
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import DOMAIN, SIGNAL_SABNZBD_UPDATED
-from ...config_entries import ConfigEntry
-from ...const import DATA_GIGABYTES, DATA_MEGABYTES, DATA_RATE_MEGABYTES_PER_SECOND
-from ...core import HomeAssistant
-from ...helpers.device_registry import DeviceEntryType
-from ...helpers.entity import DeviceInfo
-from ...helpers.entity_platform import AddEntitiesCallback
 from .const import DEFAULT_NAME, KEY_API_DATA, KEY_NAME
 
 
@@ -32,13 +36,15 @@ class SabnzbdSensorEntityDescription(SensorEntityDescription, SabnzbdRequiredKey
     """Describes Sabnzbd sensor entity."""
 
 
+SPEED_KEY = "kbpersec"
+
 SENSOR_TYPES: tuple[SabnzbdSensorEntityDescription, ...] = (
     SabnzbdSensorEntityDescription(
         key="status",
         name="Status",
     ),
     SabnzbdSensorEntityDescription(
-        key="kbpersec",
+        key=SPEED_KEY,
         name="Speed",
         native_unit_of_measurement=DATA_RATE_MEGABYTES_PER_SECOND,
         state_class=SensorStateClass.MEASUREMENT,
@@ -101,7 +107,19 @@ SENSOR_TYPES: tuple[SabnzbdSensorEntityDescription, ...] = (
     ),
 )
 
-SENSOR_KEYS: list[str] = [desc.key for desc in SENSOR_TYPES]
+OLD_SENSOR_KEYS = [
+    "current_status",
+    "speed",
+    "queue_size",
+    "queue_remaining",
+    "disk_size",
+    "disk_free",
+    "queue_count",
+    "day_size",
+    "week_size",
+    "month_size",
+    "total_size",
+]
 
 
 async def async_setup_entry(
@@ -111,11 +129,16 @@ async def async_setup_entry(
 ) -> None:
     """Set up a Sabnzbd sensor entry."""
 
-    sab_api_data = hass.data[DOMAIN][config_entry.entry_id][KEY_API_DATA]
-    client_name = hass.data[DOMAIN][config_entry.entry_id][KEY_NAME]
+    entry_id = config_entry.entry_id
+
+    sab_api_data = hass.data[DOMAIN][entry_id][KEY_API_DATA]
+    client_name = hass.data[DOMAIN][entry_id][KEY_NAME]
 
     async_add_entities(
-        [SabnzbdSensor(sab_api_data, client_name, sensor) for sensor in SENSOR_TYPES]
+        [
+            SabnzbdSensor(sab_api_data, client_name, sensor, entry_id)
+            for sensor in SENSOR_TYPES
+        ]
     )
 
 
@@ -126,21 +149,25 @@ class SabnzbdSensor(SensorEntity):
     _attr_should_poll = False
 
     def __init__(
-        self, sabnzbd_api_data, client_name, description: SabnzbdSensorEntityDescription
+        self,
+        sabnzbd_api_data,
+        client_name,
+        description: SabnzbdSensorEntityDescription,
+        entry_id,
     ):
         """Initialize the sensor."""
-        unique_id = description.key
-        self._attr_unique_id = unique_id
+
+        self._attr_unique_id = f"{entry_id}_{description.key}"
         self.entity_description = description
         self._sabnzbd_api = sabnzbd_api_data
         self._attr_name = f"{client_name} {description.name}"
         self._attr_device_info = DeviceInfo(
             entry_type=DeviceEntryType.SERVICE,
-            identifiers={(DOMAIN, DOMAIN)},
+            identifiers={(DOMAIN, entry_id)},
             name=DEFAULT_NAME,
         )
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Call when entity about to be added to hass."""
         self.async_on_remove(
             async_dispatcher_connect(
@@ -154,9 +181,11 @@ class SabnzbdSensor(SensorEntity):
             self.entity_description.key
         )
 
-        if self.entity_description.key == "speed":
-            self._attr_native_value = round(float(self._attr_native_value) / 1024, 1)
-        elif "size" in self.entity_description.key:
-            self._attr_native_value = round(float(self._attr_native_value), 2)
-
+        if self._attr_native_value is not None:
+            if self.entity_description.key == SPEED_KEY:
+                self._attr_native_value = round(
+                    float(self._attr_native_value) / 1024, 1
+                )
+            elif "size" in self.entity_description.key:
+                self._attr_native_value = round(float(self._attr_native_value), 2)
         self.schedule_update_ha_state()
